!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calculates volume integrals using Gaussian quadrature over radial coordinate.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine ma00aa(Nquad)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, one
  use numerical, only : vsmall
  use fileunits, only : ounit
  use inputlist, only : Wma00aa, Ni, Nofe
  use cputiming, only : Tma00aa
  use allglobal, only : myid, numprocs, cpus, &                          ! 
                        mn, im, in,  ll, &                               ! Fourier resolution, volume index;
                        agss, agst, agsz, agtt, agtz, agzz, agoo, &      ! averaged metric quantities;
                        igss, igst, igsz, igtt, igtz, igzz, igoo, &      ! geometric information;
                        sgss, sgst, sgsz, sgtt, sgtz, sgzz, sgoo, &      ! geometric information; innermost volume;
                        felr, lelr, ss, dss                              ! finite element basis functions, radial subgrid;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in) :: Nquad ! convenient to have this global variable as an argument to eliminate allocation statements;

  INTEGER :: ii, il, ip, ir, iq, ij, ik, jj, ipq, ipqmo, ipqmt, id01bcf, jk, kj, itype, io, oi, mjk, mjtp, mktq, imj, imk, lmj, lmk
  REAL    :: gwt(1:Nquad), abscis(1:Nquad) ! Gaussian integration weights and abscissae;

  REAL    :: lss, aa, bb, cc, dd, gf(1:3), fac(1:3), smj(-1:0), smk(-1:0)

  BEGIN(ma00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wma00aa ) then ; cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : ll="i3" ;")')cput-cpus,ll
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  do ip = 0 , Nofe
   lelr(0:1,ip,0,1:Nquad) = felr(0:1,ip,0,1:Nquad) * dss(ll,ip  )
   lelr(0:1,ip,1,1:Nquad) = felr(0:1,ip,1,1:Nquad) * dss(ll,ip-1)
  enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \heading{geometric information required in annular volumes}
!latex \item The geometric information is defined
!latex \be \mbox{\type{igss(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}  \braket{s_j}{\bar g_{ss}}{s_k} ,\\
!latex     \mbox{\type{igst(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}' \braket{s_j}{\bar g_{st}}{c_k} ,\\
!latex     \mbox{\type{igsz(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}' \braket{s_j}{\bar g_{sz}}{c_k} ,\\
!latex     \mbox{\type{igtt(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}' \varphi_{r,q}' \braket{c_j}{\bar g_{tt}}{c_k} ,\\
!latex     \mbox{\type{igtz(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}' \varphi_{r,q}' \braket{c_j}{\bar g_{tz}}{c_k} ,\\
!latex     \mbox{\type{igzz(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}' \varphi_{r,q}' \braket{c_j}{\bar g_{zz}}{c_k} ,\\
!latex     \mbox{\type{igoo(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}' \braket{c_j}{     1     }{c_k} ,\\
!latex \ee
!latex where $l=0,1$, $p=0,$\verb+Nofe+, $r=0,1$, $q=0,$\verb+Nofe+, $j,k=1,$\verb+mn+.
!latex Note that the \verb+j,k+ dependence has been compressed into a single index, \verb+jk=1,mn**2+.

!latex \be \mbox{\type{sgss(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)  \left(s^{m_k+0,2}\varphi_{r,q}\right)  \braket{s_j}{\bar g_{ss}}{s_k} ,\\
!latex     \mbox{\type{sgst(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)  \left(s^{m_k+0,2}\varphi_{r,q}\right)' \braket{s_j}{\bar g_{st}}{c_k} ,\\
!latex     \mbox{\type{sgsz(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)  \left(s^{m_k+0,2}\varphi_{r,q}\right)' \braket{s_j}{\bar g_{sz}}{c_k} ,\\
!latex     \mbox{\type{sgtt(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)' \left(s^{m_k+0,2}\varphi_{r,q}\right)' \braket{c_j}{\bar g_{tt}}{c_k} ,\\
!latex     \mbox{\type{sgtz(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)' \left(s^{m_k+0,2}\varphi_{r,q}\right)' \braket{c_j}{\bar g_{tz}}{c_k} ,\\
!latex     \mbox{\type{sgzz(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)' \left(s^{m_k+0,2}\varphi_{r,q}\right)' \braket{c_j}{\bar g_{zz}}{c_k} ,\\
!latex     \mbox{\type{sgoo(1:4,l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \left(s^{m_j+0,2}\varphi_{l,p}\right)  \left(s^{m_k+0,2}\varphi_{r,q}\right)' \braket{c_j}{     1     }{c_k} ,\\
!latex \ee

!latex \item The quantities \verb+agss(jk)+ $\equiv \ds \braket{s_j}{\bar g_{ss}}{s_k}$ etc. are computed in \verb+me00ab+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{logical control}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do ii = 1 , Ni(ll) ! loop over radial sub-grid;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item Within each sub-grid region, the NAG routine \verb+D01CBF+ is called to determine the Gaussian integration abscissae.
!latex The integrals are approximated $\int_a^b f dx \approx \sum_i w_i f(x_i)$.
   
   itype = 0 ; aa = ss(ll)%s(ii-1) ; bb = ss(ll)%s(ii) ; cc = zero ; dd = zero
   
   id01bcf = 0
   CALL(D01BCF , ( itype , aa , bb , cc , dd , Nquad , gwt(1:Nquad) , abscis(1:Nquad) , id01bcf) ) ! sets gaussian weights & abscis;
   
   FATALMESS(ma00aa,id01bcf.ne.0,failed to construct Gaussian integration abscisae and weights)
   
   if( Wma00aa ) then
   !cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : myid="i3" ; ll="i3" ; ii="i3" ; abs="99f10.6)')cput-cpus,myid,ll,ii,(abscis(1:Nquad)-aa)/(bb-aa)
    cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : myid="i3" ; ll="i3" ; ii="i3" ; abs="99f10.6)')cput-cpus,myid,ll,ii,abscis(1:Nquad)
   endif
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item The routine \verb+me00ab+ is called on the sub-sub-grid (i.e. the Gaussian abiscissae) for the evaluation of \verb+agss(j,k)+, etc.
   
   do jj = 1 , Nquad ! loop over radial sub-sub-grid (numerical quadrature);
    
    lss = abscis(jj) ! radial sub-sub-grid determined by Gassian integration;

    ADDCPU(ma00aa) ; CALL(me00ab,(lss)) ; RESETCPU ! calculate averaged `braket' quantities on radial sub-sub-grid; returns agss(1:mn,1:mn), etc.
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
    do ip = 0 , Nofe
     do iq = 0 , Nofe ! ; ipq = ip+iq ; ipqmo = ipq-1 ; ipqmt = ipq-2
      
      do il = 0 , 1 ! note that igaa, igab, igba and igbb do not depend on il, ir;
       do ir = 0 , 1
        
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

        gf(1) = gwt(jj) * lelr(il,ip,0,jj) * lelr(ir,iq,0,jj) ! note that neither    arguments are the differentiated basis function;
        gf(2) = gwt(jj) * lelr(il,ip,0,jj) * lelr(ir,iq,1,jj) ! note that the second argument  is  the differentiated basis function;
        gf(3) = gwt(jj) * lelr(il,ip,1,jj) * lelr(ir,iq,1,jj) ! note that both       arguments are the differentiated basis function;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
        
        do ij = 1 , mn
         do ik = 1 , mn ; jk = ik + (ij-1) * mn ; kj = ij + (ik-1) * mn

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
          
          if( jj.eq.1 ) then ! initialize summation quantities;

           igss(il,ip,ir,iq,jk)%s(ii) = zero
           igst(il,ip,ir,iq,jk)%s(ii) = zero
           igsz(il,ip,ir,iq,jk)%s(ii) = zero
           igtt(il,ip,ir,iq,jk)%s(ii) = zero
           igtz(il,ip,ir,iq,jk)%s(ii) = zero
           igzz(il,ip,ir,iq,jk)%s(ii) = zero
           igoo(il,ip,ir,iq,jk)%s(ii) = zero

           if( ll.eq.1 ) then ! initialize summation quantities;
            do imj = 0 , 1
             do imk = 0 , 1 ; mjk = 1 + imj + 2*imk

              sgss(mjk,il,ip,ir,iq,jk)%s(ii) = zero
              sgst(mjk,il,ip,ir,iq,jk)%s(ii) = zero
              sgsz(mjk,il,ip,ir,iq,jk)%s(ii) = zero
              sgtt(mjk,il,ip,ir,iq,jk)%s(ii) = zero
              sgtz(mjk,il,ip,ir,iq,jk)%s(ii) = zero
              sgzz(mjk,il,ip,ir,iq,jk)%s(ii) = zero
              sgoo(mjk,il,ip,ir,iq,jk)%s(ii) = zero

             enddo
            enddo
           endif ! end of if( ll.eq.1 );

          endif ! end of if( jj.eq.1 );

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

          igss(il,ip,ir,iq,jk)%s(ii) = igss(il,ip,ir,iq,jk)%s(ii) + gf(1) * agss(jk)
          igst(il,ip,ir,iq,jk)%s(ii) = igst(il,ip,ir,iq,jk)%s(ii) + gf(2) * agst(jk)
          igsz(il,ip,ir,iq,jk)%s(ii) = igsz(il,ip,ir,iq,jk)%s(ii) + gf(2) * agsz(jk)
          igtt(il,ip,ir,iq,jk)%s(ii) = igtt(il,ip,ir,iq,jk)%s(ii) + gf(3) * agtt(jk)
          igtz(il,ip,ir,iq,jk)%s(ii) = igtz(il,ip,ir,iq,jk)%s(ii) + gf(3) * agtz(jk)
          igzz(il,ip,ir,iq,jk)%s(ii) = igzz(il,ip,ir,iq,jk)%s(ii) + gf(3) * agzz(jk)
          igoo(il,ip,ir,iq,jk)%s(ii) = igoo(il,ip,ir,iq,jk)%s(ii) + gf(2) * agoo(jk)

          if( ll.eq.1 ) then
           
           do imj = 0 , 1 ; lmj = im(ij) + 2*imj ; smj(-1:0) = (/ lss**(lmj-1) , lss**(lmj) /)
            do imk = 0 , 1 ; lmk = im(ik) + 2*imk ; smk(-1:0) = (/ lss**(lmk-1) , lss**(lmk) /)
             
             mjk = 1 + imj + 2*imk
             
             sgss(mjk,il,ip,ir,iq,jk)%s(ii) = sgss(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agss(jk) * &
           (       smj( 0) * lelr(il,ip,0,jj)                              ) * (       smk( 0) * lelr(ir,iq,0,jj)                              )
             
             sgst(mjk,il,ip,ir,iq,jk)%s(ii) = sgst(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agst(jk) * &
           (       smj( 0) * lelr(il,ip,0,jj)                              ) * ( lmk * smk(-1) * lelr(ir,iq,0,jj) + smk( 0) * lelr(ir,iq,1,jj) )

             sgsz(mjk,il,ip,ir,iq,jk)%s(ii) = sgsz(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agsz(jk) * &
           (       smj( 0) * lelr(il,ip,0,jj)                              ) * ( lmk * smk(-1) * lelr(ir,iq,0,jj) + smk( 0) * lelr(ir,iq,1,jj) ) 

             sgtt(mjk,il,ip,ir,iq,jk)%s(ii) = sgtt(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agtt(jk) * &
           ( lmj * smj(-1) * lelr(il,ip,0,jj) + smj( 0) * lelr(il,ip,1,jj) ) * ( lmk * smk(-1) * lelr(ir,iq,0,jj) + smk( 0) * lelr(ir,iq,1,jj) ) 

             sgtz(mjk,il,ip,ir,iq,jk)%s(ii) = sgtz(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agtz(jk) * &
           ( lmj * smj(-1) * lelr(il,ip,0,jj) + smj( 0) * lelr(il,ip,1,jj) ) * ( lmk * smk(-1) * lelr(ir,iq,0,jj) + smk( 0) * lelr(ir,iq,1,jj) ) 

             sgzz(mjk,il,ip,ir,iq,jk)%s(ii) = sgzz(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agzz(jk) * &
           ( lmj * smj(-1) * lelr(il,ip,0,jj) + smj( 0) * lelr(il,ip,1,jj) ) * ( lmk * smk(-1) * lelr(ir,iq,0,jj) + smk( 0) * lelr(ir,iq,1,jj) ) 

             sgoo(mjk,il,ip,ir,iq,jk)%s(ii) = sgoo(mjk,il,ip,ir,iq,jk)%s(ii) + gwt(jj) * agoo(jk) * &
           (       smj( 0) * lelr(il,ip,0,jj)                              ) * ( lmk * smk(-1) * lelr(ir,iq,0,jj) + smk( 0) * lelr(ir,iq,1,jj) ) 

            enddo ! end of do imk;
           enddo ! end of do imj;
           
          endif ! end of if( ll.eq.1 );
           
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
          
         enddo ! end of do ik;
        enddo ! end of do ij;
        
       enddo ! end of do iq;
      enddo ! end of do ir;
     enddo ! end of do ip;
    enddo ! end of do il;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   enddo ! end of do jj; loop over radial sub-sub-grid; this is the summation loop;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of do ii; loop over radial sub-grid;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(ma00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine ma00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
