!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Locates periodic orbits and their residues, 
!latex on the symmetry line $\theta=0$ by integrating along the magnetic field. 
!latex \item Should extend this routine to find both symmetric orbits, ie. both the O and X points
!latex (to do this, will need to consider additional symmetry lines . . .); also, 
!latex should only be required to follow symmetric orbits for half their full periodic distance.
!latex (For high-order orbits in strongly chaotic fields, better techniques will be required, eg. \cite{Hudson_06a}.)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

module periodicitylength
  REAL :: zst,ph0
end module periodicitylength

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine pq00aa( p , q , low , Tsti , upp , res , its , err , s , t , pqfail )

  use constants
  use fileunits, only : ounit
  use inputlist, only : Nfp,odetol,Mpqits,Wpq00aa
  use cputiming
  use allglobal, only : myid,pi2nfp,cpus,ll

  use periodicitylength

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in)    :: p, q                     ! periodicity;
  REAL   , intent(in)    :: low, upp                 ! lower/upper bounds for search;
  REAL   , intent(inout) :: Tsti(6)                  ! radial & poloidal coordinates; tangent map;
  REAL   , intent(out)   :: res, err, s(0:q), t(0:q) ! residue, error, (s,t) along periodic orbit;
  INTEGER, intent(out)   :: its                      ! iterations required;
  INTEGER, intent(inout) :: pqfail

  LOGICAL   :: Leven, Loddd
  INTEGER   :: Node, i, ifail, ioe
  REAL      :: zend, tol, Tst(6), TWk(20*6), terr, snew, det, harvest, th0
  CHARACTER :: R

  EXTERNAL :: bf00aa,Tbfout,Tbfend

  BEGIN(pq00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  its = 0 ; terr = zero ; pqfail = 0

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( (q/2)*2.eq.q ) Leven = .true.
  if( (q/2)*2.ne.q ) Loddd = .true.
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Stellarator symmetry is assumed, in that only periodic orbits along $\theta=0$ line are located. 
!latex The condition that the trajectory is periodic is
!latex \be \theta( 2 \pi q)=\theta(0)+2 \pi p, \ee
!latex which must be satisfied to within \verb!q*odetol!.
!latex \item Trajectories are determined by field line following, with the field (and tangent map) provided by \verb!bf00aa!.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Node = 6 ; tol = odetol ; R = 'D' ; TWk = zero ! prepare integration variables;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  if( Tsti(1).le.low .or. Tsti(1).ge.upp) then ! this is really to catch errors in calling routine;
   write(ounit,1002)cput-cpus,myid,ll,p,q,p*one/q,its,abs(terr),low,Tsti(1),upp
   pqfail = 1
   goto 9999
  !RETURN(pq00aa)
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do ioe = 0,0 ! loop over minimax,minimizing periodic orbit;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   its = 0 ! iteration counter;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
   do ; its = its+1 ! iterate until periodic orbit is located;

    if( ioe.eq.0 ) then ; th0 = zero ; ph0 = zero
    else
     if( Loddd )          th0 =  pi  ; ph0 = zero  
     if( Leven )          th0 = zero ; ph0 =  pi 
    endif
    
    Tst = (/ Tsti(1) , th0 , one , zero , zero , one /) ! constrained to search along theta=zero;
    zst = ph0

    if( ioe.eq.0 ) then ; t(0) = Tst(2) ; s(0) = Tst(1) ! this information is returned, ultimately a smooth curve is fit;
    endif

    do i = 1,q ! integrate along magnetic field lines;
     zend = zst + pi2nfp ; ifail=1
     call D02BJF( zst , zend , Node , Tst , bf00aa , tol , R , Tbfout , Tbfend , TWk , ifail )
     
!     if( ifail.ne.0 ) then
!      pqfail=1 ; terr=one ; res=99.99
!      cput = MPI_WTIME()
!      write(ounit,1002) cput-cpus , p , q , p*one/q , its , abs(terr) , low , Tsti(1) , upp !, abs(det-one) , res

!      RETURN(pq00aa)
!     endif
     
     if( ioe.eq.0 ) then ; t(i)=Tst(2) ; s(i)=Tst(1)
     endif
     
    enddo

    if( Wpq00aa ) then
     cput = MPI_WTIME()
     write(ounit,1002)cput-cpus,myid,ll,p,q,p*one/q,its,abs(terr),low,Tsti(1),upp,abs(det-one),res,(Tst(2)-th0)/(q*pi2nfp),cput-cpui
    endif
    
    terr = Tst(2) - ( th0+p*pi2 ) ; err=abs(terr) ! what is the real transform?
    
! half-period --> full-period tangent map;
   !tm(1,1)=Tst(6) ; tm(1,2)=Tst(5) ; tm(2,1)=Tst(4) ; tm(2,2)=Tst(3) ; it=tm ; it(1,1)=tm(2,2) ; it(2,2)=tm(1,1)
   !tm=matmul(it,tm) 
   !det=tm(1,1)*tm(2,2)-tm(1,2)*tm(2,1) ; res=(two-tm(1,1)-tm(2,2))/four 

    det = Tst(3)*Tst(6) - Tst(4)*Tst(5) ; res = ( two - (Tst(3)+Tst(6)) ) / four ; err = sqrt( terr**2 + (Tst(1)-Tsti(1))**2 )
   
    cput = MPI_WTIME()

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The maximum iterations is limited by the input parameter \verb!Mpqits!. 
!latex If the periodic orbit is not located in less than \verb!Mpqits! iterations, the residue \verb!res=99.99999! is returned as an error flag.

    if( abs(terr).lt.q*odetol ) exit ! finished searching for periodic orbit; this should really be pqtol?

    if( its.ge.Mpqits ) then ; res=99.99999 ; exit ! this is essentially an error flag;
    endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item A standard Newton search, using the tangent map to provide $\partial_s \theta(2\pi q)$,  
!latex is performed to locate periodic orbits. 

    snew = Tsti(1) - terr / Tst(5) ! tm(2,1) ! Tst(5) ! Newton correction;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex If the Newton correction takes the radial guess outside [\verb!low,upp!], 
!l!atex the new guess is placed at the midpoint of the previous guess and the violated boundary.
!latex the new guess is placed randomly.

    if( snew .ge. low .and. snew .le. upp ) then ;                               Tsti(1) = snew                    ! accept Newton correction;
    else                                         ; call random_number(harvest) ; Tsti(1) = low + (upp-low)*harvest ! randomly restart;
    endif
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   enddo ! end of do ; its=its+1 loop;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   cput = MPI_WTIME()
   write(ounit,1002)cput-cpus,myid,ll,p,q,p*one/q,its,abs(terr),low,Tsti(1),upp,abs(det-one),res,(Tst(2)-th0)/(q*pi2nfp),cput-cpui
   
   if( Wpq00aa .and. abs(terr).gt.q*odetol ) pause

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  enddo ! end of do ioe=0,1 loop over minimizing,minimax periodic orbit;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(pq00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

1002 format("pq00aa : "f10.2" : myid="i3" ; ll="i3" ; "i3" /"i4" ="f9.6" ; its="i3" ; err="es7.0" ; [l,s,u]=["3f9.6" ] ;":" |det-1|="es7.0" ; R="f15.9" ; "f15.10" ; time="f10.2"s ;")
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine pq00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine Tbfout(iz,Tst) ! Poincare plot;

  use constants
  use fileunits , only :
  use inputlist , only : Nfp
  use allglobal , only : pi2nfp

  implicit none

  REAL, intent(inout) :: iz ! toroidal (integration) variable;
  REAL, intent(in) :: Tst(6)

  iz = iz + pi2nfp        ! every period ! no output required (at present);

  return

end subroutine Tbfout

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

REAL function Tbfend( iz , Tst ) ! termination of field line integration;

  use constants
  use fileunits , only :
  use inputlist , only :
  use allglobal , only : pi2nfp

  use periodicitylength

  implicit none

  REAL, intent(in) :: iz,Tst(6)

  Tbfend = iz - ( zst + pi2nfp ) 

  return

end function Tbfend

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


