!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Constructs straight field line coordinates on interfaces, 
!latex and calculates rotational transform for an arbitrary tangential field.

!latex \heading{constructing straight field line angle on interfaces}

!latex \item The algorithm stems from introducing a straight field line angle $\theta_s=\theta+\lambda(\theta,\zeta)$, 
!latex where $\lambda=\sum_i \lambda_{i}\sin(m_i\theta-n_i\zeta)$ and insisting that
!latex \be \iotabar = \frac{{\bf B}\cdot \nabla \theta_s}{{\bf B}\cdot \nabla \zeta} 
!latex = \dot \theta ( 1 + \lambda_\theta) + \lambda_\zeta, \ee
!latex where $\iotabar$ is a constant (as yet unknown).

!latex Writing $\dot \theta = - \partial_s A_\zeta / \partial_s A_\theta$, we have
!latex \be \partial_s A_\theta \, \iotabar + \partial_s A_\zeta \, \lambda_\theta - \partial_s A_\theta \, \lambda_\zeta = - \partial_s A_\zeta.
!latex \label{eq:sfla} \ee
!latex By equating coefficients of the Fourier harmonics, an equation of the form ${\bf A} \cdot {\bf x} = {\bf b}$ is obtained,
!latex where ${\bf x}=(\iotabar,\lambda_1, \lambda_2,\dots)^T$.
!latex \item The Fourier harmonics of \Eqn{sfla} are computed by double angle formulae.
!latex \item The solution to the linear system is provided by NAG \verb+f04atf+.

!latex \item Note : the Fourier resolution of the angle transformation defined by $\lambda$ may need to be quite large.
!latex The resolution of the angle transformation is defined by \verb+iMpol+ and \verb+iNtor+ which are given on input.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine tr00ab( Nvol, mns, iota ) ! construct straight-field line magnetic coordinates;

  use constants, only : zero, half
  use numerical, only : small
  use fileunits, only : ounit
  use inputlist, only : Ni, Nofe, Wtr00ab
  use cputiming, only : Ttr00ab
  use allglobal, only : cpus, myid, mn, im, in, ss, ims, ins, ll, At, Az, Atmn, Azmn, sflangle

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS 

  INTEGER, intent(in) :: Nvol, mns

  REAL,  intent(out) :: iota(0:1)

  INTEGER            :: if04atf, imn, jmn, kmn, Nlin, IA, IAA, isurf, ii, ma, na, ms, ns
  REAL               :: lcpu(0:2), lss

  REAL               :: MAT(1:mns,1:mns), RHS(1:mns) ! matrix and right hand side;
  REAL               :: AA(1:mns,1:mns), WKS1(1:mns), WKS2(1:mns) ! dummy workspace required for F04ATF;

  BEGIN(tr00ab)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wtr00ab ) then ; cput = MPI_WTIME() ; write(ounit,'("tr00ab : "f10.2" : ll="i3" ;")')cput-cpus,ll
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Note that the radial derivatives of the vector potential are required.
!latex Presently, the routine only works if \verb+Nofe.gt.0+, so that cubic or higher order radial basis functions are used to describe the vector potential.
!latex However, for the linear case (where radial gradients are not really defined on the sub-radial grid) one could take the `inside limit'.

  FATAL(tr00ab,Nofe.lt.1)

  iota(0:1) = zero ! set default intent out;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Nlin = mns ; IA = Nlin ; IAA = IA

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do isurf = 0,1 ; ii = isurf*Ni(ll) ! loop over inner and outer interfaces;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{transform-on-axis}
!latex \item For the interior interface of the innermost volume, which is the degenerate interface, i.e. the coordinate axis,
!latex the following {\em very-crude} approximation is employed.
!latex \item The near axis form for the vector potential is described in \verb+ma01ae+, from which we see
!latex \be \sqrt g {\bf B}\cdot\nabla s = \sum_{j,p} A_{\z,j,0,p}s^{m_j+2p}(-m_j)\sin(m_j\t-n_j\z) - \sum_{j,p} A_{\t,j,0,p}s^{m_j+2+2p}(+n_j)\sin(m_j\t-n_j\z).
!latex \ee
!latex Near $s=0$, this is approximately $\sqrt g {\bf B}\cdot\nabla s = - \sqrt g \sum_j A_{\z,j,0,0} \sin(m_j\t-n_j\z)$ for $m_j=1$.
!latex If $A_{\z,j,0,p}=0$ for $m_j=1$ then near the coordinate axis, the field is tangential to the coordinate surfaces, i.e. the coordinate axis and the magnetic axis coincide.

!latex \item Assuming $A_{\z,j,0,p}=0$ for $m_j=1$, to leading order the {\em angle-averaged} poloidal field is
!latex $\left< \sqrt g {\bf B}\cdot\nabla \t \right> \approx - \left< \sqrt g \right> A_{\z,1,0,1} 2 s $.

!latex \item The leading order {\em averaged} toroidal field is $\left< \sqrt g {\bf B}\cdot\nabla \z \right> \approx \left< \sqrt g \right> A_{\t,1,0,0} 2 s$.

!latex \item The final approximation for the rotational-transform on axis is $\iotabar \approx - A_{\z,1,0,1} / A_{\t,1,0,0}$.

!   if( ll.eq.1 .and. isurf.eq.0 ) then
!    iota(0) = - half * Az(ll,1,2)%s(0) / At(ll,1,0)%s(0) ! very crude transform-on-axis;
!    cycle ! dont construct straight field line angle on coordinate axis;
!   endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   lss = max( ss(ll)%s(ii) , ss(1)%s(1) )
   
   if( Wtr00ab ) then ; cput = MPI_WTIME() ; write(ounit,'("tr00ab : "f10.2" : ll="i3" ; isurf="i1"; lss="es23.15" ; calling sa00aa ;")')cput-cpus,ll,isurf,lss
   endif

   CALL(sa00aa,( lss )) ! interpolate Fourier harmonics of vector potential;
   
   if( Wtr00ab ) then ; cput = MPI_WTIME() ; write(ounit,'("tr00ab : "f10.2" : ll="i3" ; isurf="i1"; lss="es23.15" ; called  sa00aa ;")')cput-cpus,ll,isurf,lss
   endif
   
   lcpu(0)=MPI_WTIME()
   
   MAT(1:mns,1:mns) = zero ; RHS(1:mns) = zero

   do kmn = 1,mns
    
    do imn = 1,mn
     if( im(imn).eq.ims(kmn) .and. in(imn).eq.ins(kmn) ) RHS(kmn)   = RHS(kmn)   - Azmn(1,imn) ! Az(ll,imn,1)%s(ii)
     if( im(imn).eq.ims(kmn) .and. in(imn).eq.ins(kmn) ) MAT(kmn,1) = MAT(kmn,1) + Atmn(1,imn) ! At(ll,imn,1)%s(ii)
    enddo
    
    do jmn = 1,mns ! really need only go from jmn = 2,.. as lambda is odd, so the first harmonic is zero;
     do imn = 1,mn
      
      ma = im(imn)+ims(jmn) ; na = in(imn) + ins(jmn)
      ms = im(imn)-ims(jmn) ; ns = in(imn) - ins(jmn)
      
      if(               ma .eq. ims(kmn) .and. na .eq. ins(kmn) ) then
      !MAT(kmn,jmn) = MAT(kmn,jmn) + half * ( Az(ll,imn,1)%s(ii) * ims(jmn) + At(ll,imn,1)%s(ii) * ins(jmn) )
       MAT(kmn,jmn) = MAT(kmn,jmn) + half * ( Azmn(1,imn) * ims(jmn) + Atmn(1,imn) * ins(jmn) )
      endif
      if(               ms .eq. ims(kmn) .and. ns .eq. ins(kmn) ) then
      !MAT(kmn,jmn) = MAT(kmn,jmn) + half * ( Az(ll,imn,1)%s(ii) * ims(jmn) + At(ll,imn,1)%s(ii) * ins(jmn) )
       MAT(kmn,jmn) = MAT(kmn,jmn) + half * ( Azmn(1,imn) * ims(jmn) + Atmn(1,imn) * ins(jmn) )
      endif
      
      if( ms.lt.0 .and. ms .eq.-ims(kmn) .and. ns .eq.-ins(kmn) ) then
      !MAT(kmn,jmn) = MAT(kmn,jmn) + half * ( Az(ll,imn,1)%s(ii) * ims(jmn) + At(ll,imn,1)%s(ii) * ins(jmn) )
       MAT(kmn,jmn) = MAT(kmn,jmn) + half * ( Azmn(1,imn) * ims(jmn) + Atmn(1,imn) * ins(jmn) )
      endif
      
     enddo
     
    enddo
    
   enddo
    
   lcpu(1) = MPI_WTIME()
   
   if04atf = 1 ; call F04ATF( MAT , IA , RHS , Nlin , sflangle(ll,isurf,1:mns) , AA , IAA , WKS1 , WKS2 , if04atf ) ! linear solution;
   
   FATALMESS(tr00ab,if04atf.eq.1,straight-field-line matrix : singular )
   FATALMESS(tr00ab,if04atf.eq.2,straight-field-line matrix : ill cond )
   FATALMESS(tr00ab,if04atf.eq.3,straight-field-line matrix : input err)
   FATALMESS(tr00ab,if04atf.ne.0,straight-field-line matrix :  fail    )
   
   iota(isurf) = sflangle(ll,isurf,1)

   lcpu(2) = MPI_WTIME()
   
   if( Wtr00ab ) then
    cput = MPI_WTIME()
    write(ounit,1001)cput-cpus,myid,ll,isurf,lcpu(1)-lcpu(0),lcpu(2)-lcpu(1),iota(isurf) 
    1001 format("tr00ab : "f10.2" : myid="i3" ; ll="i3" ; isurf="i2" ; time of construction/solution="2f15.10", iota="es23.15" ;")
   endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  enddo ! end of do isurf=0,1 ! inner and outer interfaces;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(tr00ab)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine tr00ab

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
